#define VERSION	"2.5.2"
#define DATE	"30 Nov 2018"

/*
 * Version 1.68		               1985	Mark Brukhartz
 * Version 2.3	        	25 Sep 1991	comp.sources.misc
 * Version 2.3-dpg-3      	07 Mar 1993	Dave Gymer
 * Version 2.3-dpg-4      	14 Mar 1993	Andrew Stevens
 * Version 2.3.5 (for Linux)    11 Dec 1993     Koen Holtman
 * Version 2.3.6 (for Linux)    31 Dec 1993     Koen Holtman
 * Version 2.3.6-dpg-1   "      12 Jan 1994     Dave Gymer
 * Version 2.3.7                11 Mar 1994     Koen Holtman
 * Version 2.4                  26 Nov 1994     Anders Baekgaard, Koen Holtman
 * Version 2.4.1                08 Feb 1995     Koen Holtman
 * Version 2.4.2                21 Jan 1996     Koen Holtman
 * Version 2.4.3                15 May 1998     Koen Holtman
 * Version 2.4.4                21 Jun 1998     Koen Holtman
 * Version 2.4.5                28 Sep 1998     Koen Holtman
 * Version 2.4.6                25 Nov 1999     Koen Holtman
 * Version 2.4.7                09 Oct 2001     Koen Holtman
 * Version 2.4.7.9beta          01 Dec 2002     Koen Holtman
 * Version 2.4.7.9beta4         04 Dec 2002     Koen Holtman
 * Version 2.4.8.beta1          14 Dec 2003     Koen Holtman
 * Version 2.5                  21 Dec 2003     Koen Holtman
 * Version 2.5.1                05 Feb 2012     Koen Holtman
 * Version 2.5.1-2              09 Aug 2012     Koen Holtman
 * Version 2.5.2                30 Nov 2018     Koen Holtman
 */
