/* This file was auto-generated by the Makefile, from the afio.1 manpage file */
struct extnode de1={ ".Z", NULL };
struct extnode de2={ ".z", &de1 };
struct extnode de3={ ".gz", &de2 };
struct extnode de4={ ".bz2", &de3 };
struct extnode de5={ ".tgz", &de4 };
struct extnode de6={ ".arc", &de5 };
struct extnode de7={ ".zip", &de6 };
struct extnode de8={ ".rar", &de7 };
struct extnode de9={ ".lzh", &de8 };
struct extnode de10={ ".lha", &de9 };
struct extnode de11={ ".uc2", &de10 };
struct extnode de12={ ".tpz", &de11 };
struct extnode de13={ ".taz", &de12 };
struct extnode de14={ ".tgz", &de13 };
struct extnode de15={ ".rpm", &de14 };
struct extnode de16={ ".zoo", &de15 };
struct extnode de17={ ".deb", &de16 };
struct extnode de18={ ".gif", &de17 };
struct extnode de19={ ".jpeg", &de18 };
struct extnode de20={ ".jpg", &de19 };
struct extnode de21={ ".tif", &de20 };
struct extnode de22={ ".tiff", &de21 };
struct extnode de23={ ".png", &de22 };
struct extnode de24={ ".pdf", &de23 };
struct extnode de25={ ".arj", &de24 };
struct extnode de26={ ".avi", &de25 };
struct extnode de27={ ".bgb", &de26 };
struct extnode de28={ ".cab", &de27 };
struct extnode de29={ ".cpn", &de28 };
struct extnode de30={ ".hqx", &de29 };
struct extnode de31={ ".jar", &de30 };
struct extnode de32={ ".mp3", &de31 };
struct extnode de33={ ".mpg", &de32 };
struct extnode de34={ ".mpq", &de33 };
struct extnode de35={ ".pic", &de34 };
struct extnode de36={ ".pkz", &de35 };
struct extnode de37={ ".psn", &de36 };
struct extnode de38={ ".sit", &de37 };
struct extnode de39={ ".ogg", &de38 };
struct extnode de40={ ".smk", &de39 };
struct extnode *compexts=&de40;
